import types
from AccessControl.Permission import Permission
from zope.interface import Interface
from zope.component import getSiteManager
from zope.publisher.interfaces.browser import IBrowserView, IDefaultBrowserLayer
from zope.viewlet.interfaces import IViewlet
from Products.Carousel.browser.viewlet import CarouselViewlet

def hasViewlet():
    """Returns true if the Carousel viewlet is already registered locally"""
    sm = getSiteManager()
    return len([reg for reg in sm.registeredAdapters() if reg.factory is CarouselViewlet]) > 0

def registerViewlet(manager):
    """Creates a local Carousel viewlet registration (and removes any existing one)"""
    unregisterViewlet()

    sm = getSiteManager()
    sm.registerAdapter(
        CarouselViewlet,
        required = (Interface, IDefaultBrowserLayer, IBrowserView, manager),
        provided = IViewlet,
        name = u'Products.Carousel.viewlet'
        )

def unregisterViewlet():
    """Removes the local Carousel viewlet registration"""
    sm = getSiteManager()
    # need to not have a generator, so that we can make modifications
    regs = list(sm.registeredAdapters())
    for reg in regs:
        if reg.factory is CarouselViewlet:
            sm.unregisterAdapter(CarouselViewlet, reg.required, reg.provided, reg.name)

def addPermissionsForRole(context, role, wanted_permissions):
    """ Add permissions for a role in the context.

    Parameters:
        @param context Portal object (portal itself, Archetypes item, any inherited from RoleManager)
        @param role role name, as a string
        @param wanted_permissions tuple of permissions (string names) to add for the role    

    All wanted_permissions lose their acquiring ability
    """

    assert type(wanted_permissions) == types.TupleType

    #print "Doing role:" + role + " perms:" + str(wanted_permissions)
    for p in context.ac_inherited_permissions(all=True):
        name, value = p[:2]
        p=Permission(name, value, context)
        roles=list(p.getRoles())

        #print "Permission:" + name + " roles " + str(roles)
        if name in wanted_permissions:
            if role not in roles:
                roles.append(role)
            p.setRoles(tuple(roles))

def removePermissionsForRole(context, role, wanted_permissions):
    """ Remove permissions for a role in the context.

    Parameters:
        @param context Portal object (portal itself, Archetypes item, any inherited from RoleManager)
        @param role role name, as a string
        @param wanted_permissions tuple of permissions (string names) to add for the role    

    All wanted_permissions lose their acquiring ability
    """        

    assert type(wanted_permissions) == types.TupleType

    #print "Doing role:" + role + " perms:" + str(wanted_permissions)
    for p in context.ac_inherited_permissions(all=True):
        name, value = p[:2]
        p=Permission(name, value, context)
        roles=list(p.getRoles())

        #print "Permission:" + name + " roles " + str(roles)
        if name in wanted_permissions:
            if role in roles:
                roles.remove(role)
            p.setRoles(tuple(roles))



"""Common configuration constants
"""

PROJECTNAME = 'Carousel'

ADD_PERMISSIONS = {
    # -*- extra stuff goes here -*-
    'Carousel Banner': 'Carousel: Add Carousel Banner',
}

CAROUSEL_ID = 'carousel'

from Products.CMFCore.permissions import setDefaultRoles
setDefaultRoles('Carousel: Manage Carousel', ('Manager', 'Site Administrator'))
setDefaultRoles(ADD_PERMISSIONS['Carousel Banner'], ('Manager',))



from zope import schema
from zope.interface import Interface
from zope.schema.vocabulary import SimpleVocabulary

from Products.Carousel import CarouselMessageFactory as _


class ICarousel(Interface):
    """
    A rotating display of banners.
    """

    def getSettings():
        """
        Returns an object that provides ICarouselSettings.
        """

    def getBanners():
        """
        Returns a list of objects that provide ICarouselBanner.
        """


class ICarouselSettings(Interface):
    """
    Settings for a Carousel.
    """

    enabled = schema.Bool(
        title=_(u'Enabled'),
        description=_(u'Uncheck this box to hide the Carousel temporarily.'),
        default=True,
    )

    banner_template = schema.Choice(
        title=_(u'Banner Type'),
        description=_(u'The banner is the part of the Carousel that rotates.'
            ' Choose Default for the standard Carousel banner.'),
        vocabulary='Products.Carousel.BannerTemplates',
    )

    banner_elements = schema.List(
        title=_('Banner Elements'),
        description=_(u'Select the elements that should be visible on the'
            u' banner. Note that custom banner types may not provide all elements.'),
        value_type=schema.Choice(
            vocabulary=SimpleVocabulary.fromItems((
                (_(u'Title'), u'title'),
                (_(u'Text'), u'text'),
                (_(u'Image'), u'image'),
            )),
        ),
        default=[u'title', u'text', u'image'],
        required=False,
    )

    width = schema.Int(
        title=_(u'Banner Width'),
        description=_(u'Enter the width of the banner in pixels. If you leave'
        ' this field blank, Carousel will use the width of the first banner.'),
        required=False,
    )

    height = schema.Int(
        title=_(u'Banner Height'),
        description=_(u'Enter the height of the banner in pixels. If you leave'
        ' this field blank, Carousel will use the height of the first banner.'),
        required=False,
    )

    pager_template = schema.Choice(
        title=_(u'Pager Type'),
        description=_(u'The pager allows user to navigate between banners.'
        ' Choose the type of pager to display, or select None for no pager.'),
        vocabulary='Products.Carousel.PagerTemplates',
    )

    element_id = schema.TextLine(
        title=_(u'Unique ID'),
        description=_(u'Enter an ID for the Carousel container element.'
        ' It can be used to apply CSS to the Carousel.'),
    )

    transition_type = schema.Choice(
        title=_(u'Transition'),
        vocabulary=SimpleVocabulary.fromItems((
            (_(u'Cross Fade'), u'fade'),
            (_(u'Slide'), u'slide'),
        )),
        default=u'fade',
    )

    transition_speed = schema.Float(
        title=_(u'Transition Speed'),
        description=_(u'Enter the speed of the transition in seconds.'),
        default=0.5,
        min=0.0,
    )

    transition_delay = schema.Float(
        title=_(u'Transition Delay'),
        description=_(u'Enter the number of seconds that Carousel should pause'
            ' between banners.'),
        default=8.0,
        min=0.0,
    )

    default_page_only = schema.Bool(
        title=_(u'Only display on the default item'),
        description=_(u'Only display the Carousel on the default item of this'
            ' folder. Otherwise, the Carousel appears on every item in'
            ' the folder.'),
        default=True,
    )

    random_order = schema.Bool(
        title=_(u'Random order'),
        description=_(u'Carousel pictures will reappear in random order on every page refresh.'),
        default=True,
    )

    lazyload = schema.Bool(
        title=_(u'Enabled lazily load carousel images'),
        description=_(u'Check this box to conserve the user and the server'
            ' bandwidth if the full carousel cycle is not shown.'),
        default=False,
    )


class ICarouselSettingsView(Interface):
    """
    Marker interface for the view that displays the Carousel settings form.
    """


class ICarouselFolder(Interface):
    """Marker for a folder that can hold Carousel banners."""


class ICarouselBanner(Interface):
    """A carousel banner which may include an image, text, and/or link."""

    def getSize(scale=None):
        """ Wraps the getSize method of the image field.
        """

    def tag(**kw):
        """ Wraps the tag method of the image field."""


class ICarouselBrowserLayer(Interface):
    """Marker applied to the request during traversal of sites that
       have Carousel installed

       Not used anymore, but here for BBB.
    """



from Acquisition import aq_base
from zope.component import queryMultiAdapter
from zope.interface import Interface
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from plone.app.layout.viewlets.common import ViewletBase

from Products.Carousel.config import CAROUSEL_ID
from Products.Carousel.interfaces import ICarousel


class CarouselViewlet(ViewletBase):
    __name__ = 'Products.Carousel.viewlet'

    index = ViewPageTemplateFile('viewlet.pt')

    def _template_for_carousel(self, name, carousel):
        """
        Returns a template for rendering the banners or pager.
        """

        template = queryMultiAdapter(
            (carousel, self.request),
            Interface,
            default=None,
            name=name.replace('@@', '')
        )

        if template:
            return template.__of__(carousel)
        return None

    def update(self):
        """
        Set the variables needed by the template.
        """

        self.available = False

        context_state = self.context.restrictedTraverse('@@plone_context_state')
        if context_state.is_structural_folder() and not context_state.is_default_page():
            folder = self.context
        else:
            folder = context_state.parent()

        if hasattr(aq_base(folder), CAROUSEL_ID):
            carousel = ICarousel(folder[CAROUSEL_ID], None)
            if not carousel:
                return
        else:
            return

        settings = carousel.getSettings()

        if not settings.enabled:
            return

        if not context_state.is_default_page():
            if settings.default_page_only:
                if folder != self.context:
                    return
                elif self.context.defaultView() != context_state.view_template_id():
                    return
            elif folder == self.context:
                    if not context_state.is_view_template():
                        return

        banners = carousel.getBanners()
        if not banners:
            return

        self.banners = self._template_for_carousel(
            settings.banner_template or u'@@banner-default',
            carousel
        )

        self.pager = self._template_for_carousel(
            settings.pager_template or u'@@pager-numbers',
            carousel
        )

        width, height = banners[0].getSize()
        self.height = settings.height or height or 0
        self.width = settings.width or width or 0
        self.transition = settings.transition_type
        self.speed = int(settings.transition_speed * 1000)
        self.delay = int(settings.transition_delay * 1000)
        self.element_id = settings.element_id
        self.available = True



from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile
from collective.googleanalytics.tracking import AnalyticsBaseTrackingPlugin

class CarouselBannerPlugin(AnalyticsBaseTrackingPlugin):
    """
    A tracking plugin to track clicks on Carousel banners.
    """

    __call__ = ViewPageTemplateFile('banner.pt')



from Acquisition import aq_base
from zope.interface import alsoProvides
from Products.Five import BrowserView
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces import ISelectableConstrainTypes
from Products.Carousel.interfaces import ICarouselFolder
from Products.Carousel.utils import addPermissionsForRole
from Products.Carousel.config import CAROUSEL_ID

class CarouselManager(BrowserView):

    def __call__(self):

        if hasattr(self.context.aq_base, CAROUSEL_ID):
            carousel = getattr(self.context, CAROUSEL_ID)
        else:
            pt = getToolByName(self.context, 'portal_types')
            newid = pt.constructContent(
                            type_name='Folder',
                            container=self.context,
                            id='carousel',
                            title='Carousel Banners'
                        )
            carousel = getattr(self.context, newid)

            # exclude the (Archetypes or Dexterity) folder from navigation
            if hasattr(aq_base(carousel), 'setExcludeFromNav'):
                carousel.setExcludeFromNav(True)
            elif hasattr(aq_base(carousel), 'exclude_from_nav'):
                carousel.exclude_from_nav = True

            # mark the new folder as a Carousel folder
            alsoProvides(carousel, ICarouselFolder)

            # make sure Carousel banners are addable within the new folder
            addPermissionsForRole(carousel, 'Manager',
                                  ('Carousel: Add Carousel Banner',))
            addPermissionsForRole(carousel, 'Site Administrator',
                                  ('Carousel: Add Carousel Banner',))

            # make sure *only* Carousel banners are addable
            aspect = ISelectableConstrainTypes(carousel)
            aspect.setConstrainTypesMode(1)
            aspect.setLocallyAllowedTypes(['Carousel Banner'])
            aspect.setImmediatelyAddableTypes(['Carousel Banner'])

            carousel.reindexObject()

        self.request.RESPONSE.redirect(
            carousel.absolute_url() + '/@@edit-carousel'
        )




"""Definition of the Carousel Banner content type
"""

from zope.interface import implements

try:
    from Products.LinguaPlone import public as atapi
except ImportError:
    from Products.Archetypes import atapi
from Products.ATContentTypes.content import link
from Products.ATContentTypes.content import schemata
from Products.ATContentTypes.configuration import zconf
from Products.validation import V_REQUIRED

from Products.Carousel import CarouselMessageFactory as _
from Products.Carousel.interfaces import ICarouselBanner
from Products.Carousel.config import PROJECTNAME


CarouselBannerSchema = link.ATLinkSchema.copy() + atapi.Schema((

    atapi.ImageField('image',
        required = False,
        storage = atapi.AnnotationStorage(),
        languageIndependent = True,
        max_size = zconf.ATNewsItem.max_image_dimension,
        sizes= {},
        validators = (('isNonEmptyFile', V_REQUIRED),
                      ('checkNewsImageMaxSize', V_REQUIRED)),
        widget = atapi.ImageWidget(
            description = \
                _(u'This image will be shown when this banner is active.'),
            label= _(u'Image'),
            show_content_type = False)
        ),

    atapi.StringField('imageUrl',
        required=False,
        searchable=False,
        default = "http://",
        widget = atapi.StringWidget(
            description = _(u'Instead of uploading an image, you may enter'
                u' the URL of an image hosted on another server.'),
            label = _(u'Image URL'),
            maxlength = '511')
        ),

    atapi.TextField('text',
        required=False,
        validators = ('isTidyHtmlWithCleanup',),
        default_output_type = 'text/x-html-safe',
        widget=atapi.RichWidget(
            label=_(u'Body'),
        ),
    ),

))

# Set storage on fields copied from ATContentTypeSchema, making sure
# they work well with the python bridge properties.

CarouselBannerSchema['title'].storage = atapi.AnnotationStorage()
CarouselBannerSchema['description'].storage = atapi.AnnotationStorage()
CarouselBannerSchema['description'].widget.visible = {
    'view': 'hidden',
    'edit': 'hidden'
}
CarouselBannerSchema['remoteUrl'].widget.label = _(u'Link URL')
CarouselBannerSchema['remoteUrl'].required = False

schemata.finalizeATCTSchema(CarouselBannerSchema, moveDiscussion=False)


class CarouselBanner(link.ATLink):
    """A carousel banner which may include an image, text, and/or link."""
    implements(ICarouselBanner)

    meta_type = portal_type = "Carousel Banner"
    schema = CarouselBannerSchema

    title = atapi.ATFieldProperty('title')
    description = atapi.ATFieldProperty('description')

    # -*- Your ATSchema to Python Property Bridges Here ... -*-

    def getSize(self, scale=None):
        "Return the width and height of the image"
        return self.getField('image').getSize(self, scale=scale)

atapi.registerType(CarouselBanner, PROJECTNAME)



from zope.interface import Interface
from zope.app.publisher.browser.viewmeta import _handle_menu
from zope.configuration.fields import GlobalInterface, Path, MessageID
from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from zope.schema import TextLine
from Products.Five.browser.metaconfigure import page
from Products.Carousel.interfaces import ICarousel

class ICarouselDirective(Interface):
    """
    A ZCML directive for register a banner or pager template for Carousel.
    """
    
    name = TextLine(
        title=u"The name of the banner or pager",
        description=u"The name shows up in URLs/paths. For example 'foo'.",
        required=True,
        )
        
    template = Path(
        title=u"The name of a template that implements the banner or pager",
        description=u"Refers to a file containing a page template (should end in"
            u"extension '.pt' or '.html').",
        required=True
        )
    
    title = MessageID(
        title=u"The browser menu label for the banner or pager",
        required=True
        )
        
    layer = GlobalInterface(
        title=u"The layer the banner or pager is declared for",
        description=u"The default layer for which the banner or pager is "
                    u"applicable. By default it is applied to all layers.",
        required=False
        )

def banner(_context, name, template, title, layer=IDefaultBrowserLayer):
    """
    Registers a banner template for use with Carousel.
    """
    
    page(_context, name, 'zope.Public', ICarousel, layer=layer,
        template=template)
        
    menu = _context.resolve('zope.app.menus.carousel_bannertemplates')
    _handle_menu(_context, menu, title, 
        [ICarousel], name, 'zope.Public', layer)
        
def pager(_context, name, template, title, layer=IDefaultBrowserLayer):
    """
    Registers a pager template for use with Carousel.
    """
    
    page(_context, name, 'zope.Public', ICarousel, layer=layer,
        template=template)
        
    menu = _context.resolve('zope.app.menus.carousel_pagertemplates')
    _handle_menu(_context, menu, title, 
        [ICarousel], name, 'zope.Public', layer)


from zope.component import getUtility
from plone.app.viewletmanager.interfaces import IViewletSettingsStorage
from plone.app.layout.viewlets.interfaces import IAboveContent, IContentViews
from Products.Carousel.utils import hasViewlet, registerViewlet
from Products.Carousel import HAS_PLONE4

if HAS_PLONE4:
    default_viewlet_manager = IAboveContent
    default_viewlet_manager_name = 'plone.abovecontent'
else:
    default_viewlet_manager = IContentViews
    default_viewlet_manager_name = 'plone.contentviews'

def configureViewlet(gscontext):
    """ Add the Carousel viewlet to the plone.contentviews viewlet manager
        as a *local* adapter, if it's not already registered for a manager.
    """
    if gscontext.readDataFile('carousel_various.txt') is None:
        # don't run this step for other profiles
        return

    if not hasViewlet():
        registerViewlet(default_viewlet_manager)
        
        # make sure it's first in the content views manager
        storage = getUtility(IViewletSettingsStorage)
        skins = getattr(storage, '_order')
        for skinname in skins:
            values = list(skins[skinname].get(default_viewlet_manager_name, []))
            values.insert(0, 'Products.Carousel.viewlet')
            storage.setOrder(default_viewlet_manager_name, skinname, tuple(values))



from time import time
from random import shuffle

from persistent import Persistent
import ExtensionClass
from zope.annotation import factory
from zope.component import adapts
from zope.interface import implements
from z3c.form import form, field, group
from z3c.form.browser.checkbox import SingleCheckBoxFieldWidget, \
    CheckBoxFieldWidget
from plone.app.z3cform.layout import FormWrapper
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.interfaces import IFolderish
from Products.ATContentTypes.interface.topic import IATTopic
from Products.Carousel.interfaces import ICarousel, ICarouselSettings, \
    ICarouselFolder, ICarouselSettingsView, ICarouselBanner
from Products.Carousel import CarouselMessageFactory as _


class Carousel(ExtensionClass.Base):
    implements(ICarousel)
    adapts(ICarouselFolder)

    def __init__(self, context):
        self.context = context

    def getSettings(self):
        """
        Returns an object that provides ICarouselSettings.
        """

        return ICarouselSettings(self.context)

    def getBanners(self):
        """
        Returns a list of objects that provide ICarouselBanner.
        """

        banner_brains = []
        if IFolderish.providedBy(self.context):
            catalog = getToolByName(self.context, 'portal_catalog')
            banner_brains = catalog.searchResults({
                'path': '/'.join(self.context.getPhysicalPath()),
                'object_provides': ICarouselBanner.__identifier__,
                'sort_on': 'getObjPositionInParent',
            })
        elif IATTopic.providedBy(self.context):
            banner_brains = self.context.queryCatalog()

        banner_objects = [b.getObject() for b in banner_brains]
        banner_objects = [b for b in banner_objects if ICarouselBanner.providedBy(b)]

        # Shuffle carousel images if needde
        if self.getSettings().random_order:
            shuffle(banner_objects)

        return banner_objects


class CarouselSettings(Persistent):
    """
    Settings for a Carousel instance.
    """

    implements(ICarouselSettings)
    adapts(ICarouselFolder)

    enabled = True
    banner_template = u'@@banner-default'
    banner_elements = [u'title', u'text', u'image']
    width = None
    height = None
    pager_template = u'@@pager-numbers'
    element_id = u'carousel-default'
    transition_type = u'fade'
    transition_speed = 0.5
    transition_delay = 8.0
    default_page_only = True
    lazyload = False
    random_order = False

    def __init__(self):
        self.element_id = u'carousel-%s' % hash(time())

CarouselSettingsFactory = factory(CarouselSettings)


class AppearanceGroup(group.Group):
    """
    Appearance options.
    """

    label = _(u'Appearance')
    fields = field.Fields(ICarouselSettings).select(
        'banner_template', 'banner_elements', 'width', 'height',
        'pager_template', 'element_id', 'lazyload', 'random_order')
    fields['banner_elements'].widgetFactory = CheckBoxFieldWidget
    fields['lazyload'].widgetFactory = SingleCheckBoxFieldWidget
    fields['random_order'].widgetFactory = SingleCheckBoxFieldWidget


class TransitionGroup(group.Group):
    """
    Transition options.
    """

    label = _(u'Transition')
    fields = field.Fields(ICarouselSettings).select(
        'transition_type', 'transition_speed', 'transition_delay')


class DisplayGroup(group.Group):
    """
    Display options.
    """

    label = _(u'Display')
    fields = field.Fields(ICarouselSettings).select(
        'enabled', 'default_page_only')
    fields['enabled'].widgetFactory = SingleCheckBoxFieldWidget
    fields['default_page_only'].widgetFactory = SingleCheckBoxFieldWidget


class CarouselSettingsForm(group.GroupForm, form.EditForm):
    """
    Form for editing Carousel settings.
    """

    label = _(u'Carousel Settings')
    groups = (AppearanceGroup, TransitionGroup, DisplayGroup,)

    def getContent(self):
        return ICarouselSettings(self.context)

CarouselSettingsForm.buttons['apply'].title = _(u'Save')


class CarouselSettingsView(FormWrapper):
    """
    View for searching and filtering ATResources.
    """

    implements(ICarouselSettingsView)

    index = ViewPageTemplateFile('settings.pt')
    form = CarouselSettingsForm



from Products.CMFCore.utils import getToolByName
from Products.Carousel.interfaces import ICarouselBanner
from Products.Carousel import CarouselMessageFactory as _

def null_upgrade_step(setup_tool):
    """
    This is a null upgrade. Use it when nothing happens
    """
    pass

def upgrade_11_to_20b1(setup_tool):
    """
    Upgrade old Carousel banners by moving the description into the new
    body field.
    """

    # Install new dependencies.
    qi = getToolByName(setup_tool, 'portal_quickinstaller')
    if qi.isProductInstallable('plone.app.z3cform') and not \
        qi.isProductInstalled('plone.app.z3cform'):
        qi.installProduct('plone.app.z3cform')

    # Update Carousel banners.
    catalog = getToolByName(setup_tool, 'portal_catalog')
    transforms = getToolByName(setup_tool, 'portal_transforms')

    banners = catalog.searchResults({
        'object_provides': ICarouselBanner.__identifier__,
    })

    for banner in banners:
        description = banner.Description
        if description:
            html = transforms.convertTo('text/html',
                description, mimetype='text/plain')
            banner.getObject().setText(html.getData(), mimetype='text/html')

    # Change the tab name to Carousel.
    actions = getToolByName(setup_tool, 'portal_actions')
    obj_actions = actions.get('object', {})
    if 'carousel' in obj_actions.keys():
        obj_actions['carousel'].title = _(u'Carousel')

def upgrade_21b3_to_10(setup_tool):
    """
    Upgrade action permission settings.
    """

    actions = getToolByName(setup_tool, 'portal_actions')
    carousel = actions.get('object', {}).get('carousel', None)
    if carousel:
        if carousel.permissions == ('Manage portal',):
            carousel.permissions = ('Carousel: Manage Carousel',)


def reimport_typeinfo(setup_tool):
    """Reimport typeinfo"""
    setup_tool.runImportStepFromProfile('profile-Products.Carousel:default', 'typeinfo', run_dependencies=False, purge_old=False)



# -*- coding: utf-8 -*-
"""
This module contains the tool of Products.Carousel
"""
from setuptools import find_packages
from setuptools import setup
import os


def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()


long_description = (
    read('README.rst')
    + '\n' +
    read('CHANGES.txt')
    + '\n' +
    read('CONTRIBUTORS.txt'))


tests_require=['zope.testing']


setup(
    name='Products.Carousel',
    version='3.0',
    description="Carousel allows you to add user-configurable rotating banners to any section of a Plone site.",
    long_description=long_description,
    # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Framework :: Plone',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Development Status :: 5 - Production/Stable'],
    keywords='plone carousel slideshow banners rotating features',
    author='Groundwire',
    author_email='davidglick@groundwire.org',
    url='https://github.com/collective/Products.Carousel',
    license='GPL',
    packages=find_packages(exclude=['ez_setup']),
    namespace_packages=['Products', ],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'setuptools',
        'zope.app.component',
        'zope.app.publisher',
        'plone.app.z3cform'],
    tests_require=tests_require,
    extras_require=dict(tests=tests_require),
    test_suite = 'Products.Carousel.tests.test_docs.test_suite',

    entry_points={
        # -*- Entry points: -*-
        'z3c.autoinclude.plugin': 'target = plone',
        },
    )



from zope.interface import implements
from zope.app.publisher.interfaces.browser import IBrowserMenu
from zope.component import getUtility
from zope.schema.vocabulary import SimpleVocabulary
from Products.Carousel.interfaces import ICarousel

class DummyCarouselFolder(object):
    implements(ICarousel)

def getContext(context):
    """
    The context for the vocabulary could be an annotation, so we need
    this wrapper to get the real context.
    """
    
    if not hasattr(context, 'REQUEST') and hasattr(context, '__parent__'):
        return context.__parent__
    return context
    
def uniqueMenuItems(items, sort_first):
    """
    Returns a list of tuples in the form (title, action) where both title
    and action are unique.
    """
    
    results = {}
    for item in items:
        if not item['title'] in results.keys() \
            and not item['action'] in results.values():
            results[item['title']] = item['action']
    return sorted(results.items(),
        key=lambda item: (item[0] != sort_first, item))

def getBannerTemplates(context):
    context = getContext(context)
    menu = getUtility(IBrowserMenu, name='carousel_bannertemplates')
    items = menu.getMenuItems(DummyCarouselFolder(), context.REQUEST)
    return SimpleVocabulary.fromItems(uniqueMenuItems(items, 'Default'))
    
def getPagerTemplates(context):
    context = getContext(context)
    menu = getUtility(IBrowserMenu, name='carousel_pagertemplates')
    items = menu.getMenuItems(DummyCarouselFolder(), context.REQUEST)
    return SimpleVocabulary.fromItems(uniqueMenuItems(items, 'None'))